package tkio.service.impl;

import com.amazonaws.services.dynamodbv2.xspec.B;
import common.model.*;
import common.repository.Account4WebRepository;
import common.repository.BackVisitRepository;
import common.repository.PaymentRepository;
import common.repository.UserRepository;
import dic.RoleEnum;
import dic.RoleTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tkio.model.Account;
import tkio.model.SalesManLeader;
import tkio.repository.AccountRepository;
import tkio.repository.SalesManLeaderRepository;
import tkio.service.TkioAccountService;
import util.ValidateUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by mxq on 2017/12/26.
 */

public class TkioAccountServiceImpl implements TkioAccountService {

    @Autowired
    Account4WebRepository account4WebRepository;

    @Autowired
    SalesManLeaderRepository salesManLeaderRepository;

    @Autowired
    AccountRepository accountRepository;

    @Autowired
    BackVisitRepository backVisitRepository;

    @Autowired
    PaymentRepository paymentRepository;

    @Override
    public List<Account4Web> findAll(User user) {
        List<Account4Web> result = new ArrayList<>();
        List<Account4Web> account4WebList = null;
        if(user.getRole().equals(RoleEnum.MANAGER.getKey())){
            account4WebList = account4WebRepository.findAll();
        } else{
            List<Integer> idList = new ArrayList<>();
            if(user.getRoleType().equals(RoleTypeEnum.MANAGER.getKey())){
                List<SalesManLeader> salesManLeaderList = salesManLeaderRepository.findByEmail(user.getEmail());
                if (ValidateUtil.isValid(salesManLeaderList)){
                    for(SalesManLeader sml : salesManLeaderList){
                        idList.add(sml.getId());
                    }
                }
            }else{
                SalesManLeader salesManLeader = salesManLeaderRepository.findOneByEmail(user.getEmail());
                idList.add(salesManLeader.getId());
            }
            if(ValidateUtil.isValid(idList)){
                List<Account> accountList = accountRepository.findBussnissMan(idList);
                List<String> emails = new ArrayList<>();
                if(ValidateUtil.isValid(accountList)){
                    for(Account ac : accountList){
                        emails.add(ac.getEmail());
                    }
                    account4WebList = account4WebRepository.findByEmails(emails);
                }
            }
        }
        if(ValidateUtil.isValid(account4WebList)){
            Map<String, Integer> backTimeMap = getBackTime();
            Map<String, Long> payMap = getPayment();
            for(Account4Web aw : account4WebList){
                if(backTimeMap.containsKey(aw.getEmail())){
                    aw.setBackTime(backTimeMap.get(aw.getEmail()));
                } else{
                    aw.setBackTime(0);
                }
                if(payMap.containsKey(aw.getEmail())){
                    aw.setMoney(payMap.get(aw.getEmail()));
                } else{
                    aw.setMoney(0L);
                }
                result.add(aw);
            }
        }
        return result;
    }
    public Map<String, Integer> getBackTime(){
        Map<String, Integer> map = new HashMap<>();
        List<BackVisit> all = backVisitRepository.findAll();
        if (ValidateUtil.isValid(all)){
            for(BackVisit bv : all){
                if(map.containsKey(bv.getEmail())){
                    map.put(bv.getEmail(), map.get(bv.getEmail()) + 1);
                } else{
                    map.put(bv.getEmail(), 1);
                }
            }
        }
        return map;
    }

    public Map<String, Long> getPayment(){
        Map<String, Long> map = new HashMap<>();
        List<Payment> all = paymentRepository.findAll();
        if (ValidateUtil.isValid(all)){
            for(Payment pa : all){
                if(map.containsKey(pa.getEmail())){
                    map.put(pa.getEmail(), map.get(pa.getEmail()) + pa.getMoney());
                } else{
                    map.put(pa.getEmail(), pa.getMoney());
                }
            }
        }
        return map;
    }
}
